# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nbox/framework/protos/model_spec.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&nbox/framework/protos/model_spec.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\x91\x01\n\tFramework\x12\x0f\n\x07package\x18\x01 \x01(\t\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x0e\n\x06method\x18\x03 \x01(\t\x12&\n\x06kwargs\x18\x04 \x03(\x0b\x32\x16.Framework.KwargsEntry\x1a-\n\x0bKwargsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"M\n\x06Tensor\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05shape\x18\x02 \x03(\x03\x12\x17\n\x0f\x64imension_names\x18\x03 \x03(\t\x12\r\n\x05\x64type\x18\x05 \x01(\t\"\x8c\x01\n\nDeployment\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x14\n\x0cworkspace_id\x18\x03 \x01(\t\x12)\n\x04type\x18\x04 \x01(\x0e\x32\x1b.Deployment.DeploymentTypes\"#\n\x0f\x44\x65ploymentTypes\x12\x10\n\x0cNBOX_SERVING\x10\x00\"\x86\x02\n\tModelSpec\x12\x1a\n\x06source\x18\x01 \x01(\x0b\x32\n.Framework\x12\x1a\n\x06target\x18\x02 \x01(\x0b\x32\n.Framework\x12\x0e\n\x06\x66older\x18\x03 \x01(\t\x12\x0c\n\x04name\x18\x04 \x01(\t\x12\n\n\x02id\x18\x05 \x01(\t\x12\x14\n\x0crequirements\x18\x0b \x03(\t\x12\x31\n\rexported_time\x18\x0c \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x17\n\x06inputs\x18\r \x03(\x0b\x32\x07.Tensor\x12\x18\n\x07outputs\x18\x0e \x03(\x0b\x32\x07.Tensor\x12\x1b\n\x06\x64\x65ploy\x18\x0f \x01(\x0b\x32\x0b.Deploymentb\x06proto3')



_FRAMEWORK = DESCRIPTOR.message_types_by_name['Framework']
_FRAMEWORK_KWARGSENTRY = _FRAMEWORK.nested_types_by_name['KwargsEntry']
_TENSOR = DESCRIPTOR.message_types_by_name['Tensor']
_DEPLOYMENT = DESCRIPTOR.message_types_by_name['Deployment']
_MODELSPEC = DESCRIPTOR.message_types_by_name['ModelSpec']
_DEPLOYMENT_DEPLOYMENTTYPES = _DEPLOYMENT.enum_types_by_name['DeploymentTypes']
Framework = _reflection.GeneratedProtocolMessageType('Framework', (_message.Message,), {

  'KwargsEntry' : _reflection.GeneratedProtocolMessageType('KwargsEntry', (_message.Message,), {
    'DESCRIPTOR' : _FRAMEWORK_KWARGSENTRY,
    '__module__' : 'nbox.framework.protos.model_spec_pb2'
    # @@protoc_insertion_point(class_scope:Framework.KwargsEntry)
    })
  ,
  'DESCRIPTOR' : _FRAMEWORK,
  '__module__' : 'nbox.framework.protos.model_spec_pb2'
  # @@protoc_insertion_point(class_scope:Framework)
  })
_sym_db.RegisterMessage(Framework)
_sym_db.RegisterMessage(Framework.KwargsEntry)

Tensor = _reflection.GeneratedProtocolMessageType('Tensor', (_message.Message,), {
  'DESCRIPTOR' : _TENSOR,
  '__module__' : 'nbox.framework.protos.model_spec_pb2'
  # @@protoc_insertion_point(class_scope:Tensor)
  })
_sym_db.RegisterMessage(Tensor)

Deployment = _reflection.GeneratedProtocolMessageType('Deployment', (_message.Message,), {
  'DESCRIPTOR' : _DEPLOYMENT,
  '__module__' : 'nbox.framework.protos.model_spec_pb2'
  # @@protoc_insertion_point(class_scope:Deployment)
  })
_sym_db.RegisterMessage(Deployment)

ModelSpec = _reflection.GeneratedProtocolMessageType('ModelSpec', (_message.Message,), {
  'DESCRIPTOR' : _MODELSPEC,
  '__module__' : 'nbox.framework.protos.model_spec_pb2'
  # @@protoc_insertion_point(class_scope:ModelSpec)
  })
_sym_db.RegisterMessage(ModelSpec)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _FRAMEWORK_KWARGSENTRY._options = None
  _FRAMEWORK_KWARGSENTRY._serialized_options = b'8\001'
  _FRAMEWORK._serialized_start=76
  _FRAMEWORK._serialized_end=221
  _FRAMEWORK_KWARGSENTRY._serialized_start=176
  _FRAMEWORK_KWARGSENTRY._serialized_end=221
  _TENSOR._serialized_start=223
  _TENSOR._serialized_end=300
  _DEPLOYMENT._serialized_start=303
  _DEPLOYMENT._serialized_end=443
  _DEPLOYMENT_DEPLOYMENTTYPES._serialized_start=408
  _DEPLOYMENT_DEPLOYMENTTYPES._serialized_end=443
  _MODELSPEC._serialized_start=446
  _MODELSPEC._serialized_end=708
# @@protoc_insertion_point(module_scope)
