# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/relics.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nbox.sublime.proto import common_pb2 as proto_dot_common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12proto/relics.proto\x1a\x12proto/common.proto\"\xa6\x01\n\x05Relic\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x12\n\ncreated_on\x18\x03 \x01(\x03\x12\x15\n\rlast_modified\x18\x04 \x01(\x03\x12\x0f\n\x07starred\x18\x05 \x01(\x08\x12\x0c\n\x04tags\x18\x06 \x03(\t\x12\x14\n\x0cworkspace_id\x18\x07 \x01(\t\x12\x19\n\x06\x61ws_s3\x18\x64 \x01(\x0b\x32\x07.AWS_S3H\x00\x42\x08\n\x06\x62ucket\"\x80\x04\n\tRelicFile\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x12\n\ncreated_on\x18\x02 \x01(\x03\x12\x15\n\rlast_modified\x18\x03 \x01(\x03\x12\x0f\n\x07starred\x18\x04 \x01(\x08\x12\x0c\n\x04tags\x18\x05 \x03(\t\x12\x0c\n\x04size\x18\x06 \x01(\x03\x12\x0f\n\x07\x63omment\x18\x07 \x01(\t\x12\x10\n\x08username\x18\x08 \x01(\t\x12\"\n\x04type\x18\t \x01(\x0e\x32\x14.RelicFile.RelicType\x12\x14\n\x0cworkspace_id\x18\n \x01(\t\x12\x12\n\nrelic_name\x18\x0b \x01(\t\x12\x10\n\x08relic_id\x18\x0c \x01(\t\x12\x0b\n\x03url\x18\x64 \x01(\t\x12(\n\x07headers\x18\x65 \x03(\x0b\x32\x17.RelicFile.HeadersEntry\x12\"\n\x04\x62ody\x18\x66 \x03(\x0b\x32\x14.RelicFile.BodyEntry\x12\x1a\n\x06\x61ws_s3\x18\x96\x01 \x01(\x0b\x32\x07.AWS_S3H\x00\x1a.\n\x0cHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a+\n\tBodyEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\",\n\tRelicType\x12\x08\n\x04\x46ILE\x10\x00\x12\n\n\x06\x46OLDER\x10\x01\x12\t\n\x05RELIC\x10\x02\x42\x08\n\x06\x62ucketb\x06proto3')



_RELIC = DESCRIPTOR.message_types_by_name['Relic']
_RELICFILE = DESCRIPTOR.message_types_by_name['RelicFile']
_RELICFILE_HEADERSENTRY = _RELICFILE.nested_types_by_name['HeadersEntry']
_RELICFILE_BODYENTRY = _RELICFILE.nested_types_by_name['BodyEntry']
_RELICFILE_RELICTYPE = _RELICFILE.enum_types_by_name['RelicType']
Relic = _reflection.GeneratedProtocolMessageType('Relic', (_message.Message,), {
  'DESCRIPTOR' : _RELIC,
  '__module__' : 'proto.relics_pb2'
  # @@protoc_insertion_point(class_scope:Relic)
  })
_sym_db.RegisterMessage(Relic)

RelicFile = _reflection.GeneratedProtocolMessageType('RelicFile', (_message.Message,), {

  'HeadersEntry' : _reflection.GeneratedProtocolMessageType('HeadersEntry', (_message.Message,), {
    'DESCRIPTOR' : _RELICFILE_HEADERSENTRY,
    '__module__' : 'proto.relics_pb2'
    # @@protoc_insertion_point(class_scope:RelicFile.HeadersEntry)
    })
  ,

  'BodyEntry' : _reflection.GeneratedProtocolMessageType('BodyEntry', (_message.Message,), {
    'DESCRIPTOR' : _RELICFILE_BODYENTRY,
    '__module__' : 'proto.relics_pb2'
    # @@protoc_insertion_point(class_scope:RelicFile.BodyEntry)
    })
  ,
  'DESCRIPTOR' : _RELICFILE,
  '__module__' : 'proto.relics_pb2'
  # @@protoc_insertion_point(class_scope:RelicFile)
  })
_sym_db.RegisterMessage(RelicFile)
_sym_db.RegisterMessage(RelicFile.HeadersEntry)
_sym_db.RegisterMessage(RelicFile.BodyEntry)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _RELICFILE_HEADERSENTRY._options = None
  _RELICFILE_HEADERSENTRY._serialized_options = b'8\001'
  _RELICFILE_BODYENTRY._options = None
  _RELICFILE_BODYENTRY._serialized_options = b'8\001'
  _RELIC._serialized_start=43
  _RELIC._serialized_end=209
  _RELICFILE._serialized_start=212
  _RELICFILE._serialized_end=724
  _RELICFILE_HEADERSENTRY._serialized_start=577
  _RELICFILE_HEADERSENTRY._serialized_end=623
  _RELICFILE_BODYENTRY._serialized_start=625
  _RELICFILE_BODYENTRY._serialized_end=668
  _RELICFILE_RELICTYPE._serialized_start=670
  _RELICFILE_RELICTYPE._serialized_end=714
# @@protoc_insertion_point(module_scope)
